@@ -52,7 +52,7 @@ def upgrade_help():
     print "http://codezen.org/canto/config/#upgrading-from-06x"
 
 class Main():
-    def __init__(self):
+    def __init__(self, stdscr = None):
         signal.signal(signal.SIGUSR2, self.debug_out)
 
         # Let locale figure itself out
@@ -87,6 +87,16 @@ class Main():
         self.cfg.log("Time: %s" % time.asctime())
         self.cfg.log("Config parsed successfully.")
 
+        # If we were passed an existing curses screen (i.e. restart)
+        # pass it through to the config.
+
+        self.cfg.stdscr = stdscr
+        if self.cfg.stdscr:
+            self.restarting = True
+        else:
+            self.restarting = False
+        self.restart = False
+
         # Default arguments.
         flags = 0 
         feed_ct = None
@@ -176,12 +186,14 @@ class Main():
         self.cfg.log("Populating feeds...")
         for f in self.cfg.feeds:
             self.ph.send((PROC_UPDATE, f.URL, []))
+        for f in self.cfg.feeds:
             f.merge(self.ph.recv()[1])
-        self.ph.send((PROC_GETTAGS, ))
 
+        self.ph.send((PROC_GETTAGS, ))
         fixedtags = self.ph.recv()
-        
+
         self.ph.kill_process()
+
         for i, f in enumerate(self.cfg.feeds):
             self.cfg.feeds[i].tags = fixedtags[i]
 
@@ -267,24 +279,25 @@ class Main():
         # the client, so we fire up ncurses and add the screen
         # information to our Cfg().
 
-        self.cfg.stdscr = curses.initscr()
-        self.cfg.stdscr.nodelay(1)
+        if not self.restarting:
+            self.cfg.stdscr = curses.initscr()
+            self.cfg.stdscr.nodelay(1)
 
-        # curs_set can return ERR, we shouldn't care
-        try:
-                curses.curs_set(0)
-        except:
-                pass
+            # curs_set can return ERR, we shouldn't care
+            try:
+                    curses.curs_set(0)
+            except:
+                    pass
 
-        # if any of these mess up though, the rest of the
-        # the operation is suspect, so die.
-        try:
-                curses.noecho()
-                curses.start_color()
-                curses.use_default_colors()
-        except:
-                self.cfg.log("Unable to init curses, bailing")
-                self.done()
+            # if any of these mess up though, the rest of the
+            # the operation is suspect, so die.
+            try:
+                    curses.noecho()
+                    curses.start_color()
+                    curses.use_default_colors()
+            except:
+                    self.cfg.log("Unable to init curses, bailing")
+                    self.done()
 
         self.sigusr = 0
         self.resize = 0
@@ -301,7 +314,7 @@ class Main():
             self.cfg.log("Unable to init curses color pairs!")
 
         self.cfg.log("Curses initialized.")
-    
+
         # Instantiate the base Gui class
         self.gui = Gui(self.cfg, self.cfg.tags.cur())
 
@@ -322,8 +335,9 @@ class Main():
         # shitting all over the terminal.
 
         try:
-            # Initial draw of the screen
-            self.refresh()
+            # Initial draw of the screen, if not restarting
+            self.refresh(self.restarting)
+            self.restarting = False
 
             # Main program loop, terminated when all handlers have
             # deregistered / exited.
@@ -372,6 +386,9 @@ class Main():
 
                     # Make sure we don't pin the CPU, so if there's no input and
                     # no waiting updates, sleep for awhile.
+                    if not self.ph.pid:
+                        time.sleep(0.01)
+                        continue
 
                     r = self.ph.recv(True, 0.01)
                     if r:
@@ -440,6 +457,9 @@ class Main():
                         self.update(1, ufds)
                     elif r == REDRAW_ALL:
                         self.gui.draw_elements()
+                    elif r == RESTART:
+                        self.restart = True
+                        self.gui = None
                     elif r == EXIT:
                         self.gui = None
                         break
@@ -453,17 +473,21 @@ class Main():
 
     def done(self, a=None, b=None):
         # Unset signals.
-        for s in [signal.SIGALRM, signal.SIGWINCH,
-                signal.SIGCHLD, signal.SIGINT]:
-            signal.signal(s, signal.SIG_IGN)
+        for s in [signal.SIGWINCH, signal.SIGCHLD, signal.SIGINT]:
+            signal.signal(s, signal.SIG_DFL)
+        signal.signal(signal.SIGALRM, signal.SIG_IGN)
 
         # Kill the message log
         self.cfg.msg = None
 
         # Kill curses
-        curses.endwin()
+        if not self.restart:
+            try:
+                curses.endwin()
+            except:
+                pass
 
-        self.cfg.log("Curses done.")
+            self.cfg.log("Curses done.")
 
         # If there was an exception, nicely print it out.
         if self.estring:
@@ -479,18 +503,7 @@ class Main():
         self.ph.sync()
         self.cfg.log("Flushed to disk.")
 
-        # This is a shitty workaround. The processing module for 2.5
-        # insists on occasionally writing an exception to
-        # stderr which is harmless but messy. To avoid, pipe
-        # stderr to /dev/null. I hope that this has been resolved in
-        # 2.6's builtin multiprocessing module, because it's really poor
-        # form for a library to output to stdout/stderr.
-
-        fd = os.open("/dev/null", os.O_RDWR)
-        os.dup2(fd, sys.stderr.fileno())
-
         self.ph.kill_process()
-        sys.exit(0)
 
     # For the most part, it's smart to avoid doing anything but set a flag in an
     # signal handler. CHLD is an exception because the only case in which we do
@@ -604,12 +617,17 @@ class Main():
     # Refresh should only be called when it's possible that the screen has
     # changed shape. 
 
-    def refresh(self):
+    def refresh(self, restart = False):
         # Get new self.cfg.{height, width}
-        curses.endwin()
-        self.cfg.stdscr.touchwin()
-        self.cfg.stdscr.refresh()
-        self.cfg.stdscr.keypad(1)
+        if not restart:
+            try:
+                curses.endwin()
+            except:
+                pass
+
+            self.cfg.stdscr.touchwin()
+            self.cfg.stdscr.refresh()
+            self.cfg.stdscr.keypad(1)
 
         self.cfg.height, self.cfg.width = self.cfg.stdscr.getmaxyx()
 
